#******************************************************************************************************
#*MERGE PROCESS AND PATSTAT DATA 
#*
#*IMPORT: process_patents_epo.txt, process_patents_uspto.txt, pp_patstat.sas7bdat
#*EXPORT: process_patstat.csv
#*NOTES: 
#*      - skript can be found on the server: 
#*           \\tsclient\C\Users\franziska.tinnefeld\OneDrive - Universit Cattolica del Sacro Cuore\Paper1\Skripts
#*
#******************************************************************************************************
# Libraries----

#sapply(c("dplyr", "magrittr", "readr", "haven"), install.packages)
library(dplyr); library(magrittr); library(readr); library(haven)
#******************************************************************************************************

# Set-up----

## Load Data----

# process data
process_patents_epo <- read.csv("D:/Tinnefeld-user6/Data/original_process_patents/process_patents_epo.txt")
process_patents_uspto <- read.csv("D:/Tinnefeld-user6/Data/original_process_patents/process_patents_uspto.txt")

# patent data
patstat <- read_sas("D:/Tinnefeld-user6/pp_patstat.sas7bdat", NULL) # 73,473,097


#******************************************************************************************************
# Merge----

## EPO and USPTO----
process <- rbind(process_patents_epo, process_patents_uspto) # 21,289,303 rows
rm(process_patents_epo, process_patents_uspto)
process$appln_id <- as.numeric(process$appln_id)
process$publn_date <- as.Date(process$publn_date)




## Process and PATSTAT----
data <- merge(patstat, process %>%    # 65,544,977 rows
                        select(-publn_auth, -publn_nr, -publn_nr_original,
                               -publn_first_grant, -appln_filing_date,
                               -earliest_filing_date, -docdb_family_id), 
              by = c("appln_id", 
                     "publn_date",
                     "publn_kind"))
rm(process, patstat)


#******************************************************************************************************
# Clean----

str(data)
data$publn_nr <- as.numeric(data$publn_nr)

#********************************************
## Date variables----

summary(data[,sapply(data, class) == c("Date")])
  # why publn_date is 1833 but earliest_filing_date 2000?
    data %>% 
         filter(publn_date < 2000) # 13 rows
          # all have earliest_filing_year=9999
          data %>% filter(earliest_filing_year == 9999) # 16 rows
              # all have appln_filing_year=9999
              data %>% filter(appln_filing_year==9999)# same 16 rows

  # drop entries with earliest_filing_year=9999
  data <- data %>% filter(earliest_filing_year != 9999) # 65,544,961 rows
summary(data[,sapply(data, class) == c("Date")])


#********************************************
## Character variables----
sapply(data[,sapply(data, class) == c("character")], unique)
  # drop receiving_office
    data <- data %>% select(-receiving_office)
  
  # is appln_auth and publn_auth always the same?
    length(which(data$appln_auth == data$publn_auth))
    length(which(data$appln_auth != data$publn_auth)) 
      # YES! therefore only keep one
      data <- data %>% 
                select(-publn_auth) %>%  # 62 variables
                rename(appln_publn_auth = appln_auth)

# create dummy variables     
  # granted
  data$granted[which(data$granted=="N")] <- 0
  data$granted[which(data$granted=="Y")] <- 1
  data$granted <- as.factor(data$granted)
  
  # publn_first_grant
  data$publn_first_grant[which(data$publn_first_grant=="N")] <- 0
  data$publn_first_grant[which(data$publn_first_grant=="Y")] <- 1
  data$publn_first_grant <- as.factor(data$publn_first_grant)
  
  # organisation_flag
  data$organisation_flag[which(data$organisation_flag=="")] <- 0
  data$organisation_flag[which(data$organisation_flag=="Y")] <- 1
  data$organisation_flag <- as.factor(data$organisation_flag)
  
  # eu_member
  data$eu_member[which(data$eu_member=="")] <- 0
  data$eu_member[which(data$eu_member=="Y")] <- 1
  data$eu_member <- as.factor(data$eu_member)
  
  # epo_member
  data$epo_member[which(data$epo_member=="")] <- 0
  data$epo_member[which(data$epo_member=="Y")] <- 1
  data$epo_member <- as.factor(data$epo_member)
  
  # oecd_member
  data$oecd_member[which(data$oecd_member=="")] <- 0
  data$oecd_member[which(data$oecd_member=="Y")] <- 1
  data$oecd_member <- as.factor(data$oecd_member)
  
  # discontinued
  data$discontinued[which(data$discontinued=="")] <- 0
  data$discontinued[which(data$discontinued=="Y")] <- 1
  data$discontinued <- as.factor(data$discontinued)
  
  # Y02
  data$Y02 <- as.numeric(data$Y02)
  data$Y02[which(data$Y02!="Y02")] <- 0 
  data$Y02[which(data$Y02=="Y02")] <- 1
  data$Y02 <- as.factor(data$Y02)

# report NAs appropriately
  # person_ctry_code
  data$person_ctry_code[which(data$person_ctry_code=="")] <- NA
  data$person_ctry_code[which(data$person_ctry_code=="--")] <- NA
  # psn_sector
  data$psn_sector[which(data$psn_sector=="")] <- NA
  # iso_alpha3
  data$iso_alpha3[which(data$iso_alpha3=="")] <- NA
  # st3_name
  data$st3_name[which(data$st3_name=="")] <- NA
  # continent
  data$continent[which(data$continent=="")] <- NA
  # Y02PN
  data$Y02PN <- gsub(" ", "", x = data$Y02PN, fixed = T)
  data$Y02PN[which(data$Y02PN=="")] <- NA
  # nace2_code
  data$nace2_code[which(data$nace2_code=="")] <- NA
  # title_process
  data$title_process[which(data$title_process=="")] <- NA
  # title_use
  data$title_use[which(data$title_use=="")] <- NA
  # abstract_process
  data$abstract_process[which(data$abstract_process=="")] <- NA
  # abstract_use
  data$abstract_use[which(data$abstract_use=="")] <- NA
  # abstract_process_use_pred
  data$abstract_process_use_pred[which(data$abstract_process_use_pred=="")] <- NA

#********************************************
## Numeric variables----

summary(data[,sapply(data,class)=="numeric"])


#********************************************
## Integer variables----
  
summary(data[,sapply(data,class)=="integer"])

  
#*********************************************  
## Factor variables----
  
summary(data[,sapply(data,class)=="factor"])
  

#******************************************************************************************************
# Save---
  
  # write_csv(readr) cos faster than write.csv(base)
  
  # save on server:
    setwd("D:/Tinnefeld-user6/Data/4_output_data") 
          # running time: 17min
    write_csv(data, file = "process_patstat.csv")    
  # save on Toshiba 4TB:
    setwd("//tsclient/E/Patstat2")
    write_csv(data, file = "process_patstat.csv")
  
  
  
  