#******************************************************************************************************
#*MERGE PWT to Process-WIOT data & get C-I level data
#*
#*IMPORT: pp_wiot.csv,
#*        pwt100.xlsx
#*        pwt100-labor-detail.dta
#*        
#*EXPORT: 6_allPatents_balancedPanel.csv
#*NOTES: - C-I data for process share, patents, penn-world tables 
#*      - skript can be found on the server: 
#*           \\tsclient\C\Users\franziska.tinnefeld\OneDrive - Universit?? Cattolica del Sacro Cuore\Paper1\Skripts
#*
#******************************************************************************************************
# Libraries----

#sapply(c("dplyr", "magrittr", "readr", "haven"), install.packages)
library(dplyr); library(magrittr); library(readr); library(haven); library(readxl); library(ggplot2)
#******************************************************************************************************

# Set-up----

## Load Data----

### pp_wiot.csv data---- 
data <- read_csv("D:/Tinnefeld-user6/Data/5_output_data/pp_wiot.csv")

# Penn World Tables----
datapwt <- read_excel("D:/Tinnefeld-user6/Data/original_penn_wt/pwt100.xlsx", 
                      sheet = "Data")
# select variables
datapwt <- datapwt %>% 
  select(year, countrycode, country, currency_unit, 
         pop, emp, hc, rgdpo, rtfpna, labsh) %>% 
  rename(country_pwt = country)

# labor data detailed - used in Reshef 2021 for different labour share measures
datalabour <- read_dta("D:/Tinnefeld-user6/Data/original_penn_wt/pwt100-labor-detail.dta")

## Merge Data----

data <- merge(data, datapwt,
                 by.x = c("iso_alpha3", "earliest_filing_year"),
                 by.y = c("countrycode", "year"),
                 all.x = TRUE)
rm(datapwt)


### Check merged data----

# check whether all non ROW countries have merged with pwt data
data %>% select(country, country_pwt, iso_alpha3, st3_name) %>% 
  filter(country != "ROW") %>% distinct() #yes they have

# exclude ROW countries
data_woROW <- data %>% filter(country!="ROW")
#number of appln_id in data excl. and incl. ROW respectively
length(unique(data_woROW$appln_id)) #6,727,248
length(unique(data$appln_id)) #6,816,366

data <- data_woROW

rm(data_woROW)

# change classes of variables
data$iso_alpha3 <- as.factor(data$iso_alpha3)
data$appln_kind <- as.factor(data$appln_kind)
data$appln_publn_auth <- as.factor(data$appln_publn_auth)

data$publn_kind <- as.factor(data$publn_kind)
data$ipr_type <- as.factor(data$ipr_type)
data$psn_sector <- as.factor(data$psn_sector)
data$person_ctry_code <- as.factor(data$person_ctry_code)
data$Y02PN <- as.factor(data$Y02PN)
data$IndustryCode <- as.factor(data$IndustryCode)
data$eu_member <- as.factor(data$eu_member)
data$epo_member <- as.factor(data$epo_member)
data$oecd_member <- as.factor(data$oecd_member)



# number of unique appln_id for which data is missing in claims
cbind(
  "type" = rbind(c("total claims"), c("indep. claims")),
  rbind(data %>% filter(is.na(total_count)) %>% select(appln_id) %>% 
          distinct() %>% count(),
        data %>% filter(is.na(total_count_ind)) %>% select(appln_id) %>% 
          distinct() %>% count()))


# for each application choose the latest publication date
df <- data %>% group_by(appln_id, earliest_filing_year) %>% 
  filter(publn_date == max(publn_date)) %>% 
  ungroup()

# only select the important columns
df <- df %>% select(earliest_filing_year, appln_id, person_ctry_code,   
                    Y02PN, weight, IndustryCode, IndustryDescription,
                    VA_share, labsh, 
                    process_ratio_2_ind, process_ratio_5_ind, use_ratio_ind, total_count_ind,
                    title_process, title_use, abstract_process, abstract_use,
                    docdb_family_size, nb_applicants, nb_inventors, 
                    person_id, psn_sector,
                    pop, emp, hc, rgdpo, rtfpna, VA, GO,
                    iso_alpha3, st3_name, eu_member, epo_member, oecd_member)

# Explore data----

# EU - 0/1
df %>% 
  group_by(eu_member, earliest_filing_year) %>% 
  select(appln_id, title_process, Y02PN) %>%
  distinct() %>% 
  count() %>% 
  ungroup() %>% 
  ggplot(mapping = aes(earliest_filing_year, n, col = as.factor(eu_member))) +
  geom_line() +
  ylab("") +
  xlab("") +
  ggtitle("Patents in WIOT countries") +
  labs(color = "EU-member") +
  theme_minimal()

# mean process_ratio_5_ind for Y02P and non Y02P
df %>% 
  select(appln_id, process_ratio_2_ind, process_ratio_5_ind, Y02PN) %>%
  mutate(Y02P = !is.na(Y02PN)) %>% 
  group_by(Y02P) %>% 
  distinct() %>% 
  summarise(mean_process_share_2 = mean(process_ratio_2_ind, na.rm = T))
# mean process_ratio_2_ind for Y02P and non Y02P
df %>% 
  select(appln_id, process_ratio_2_ind, process_ratio_5_ind, Y02PN) %>%
  mutate(Y02P = !is.na(Y02PN)) %>% 
  group_by(Y02P) %>% 
  distinct() %>% 
  summarise(mean_process_share_5 = mean(process_ratio_5_ind, na.rm = T))


# correlation of process_ratio and VA-share
print('Total')
cor(df$VA_share, df$process_ratio_2_ind, use = "complete.obs")
cor(df$VA_share, df$process_ratio_5_ind, use = "complete.obs")

print('US')
cor(df$VA_share[df$person_ctry_code == "US"], df$process_ratio_2_ind[df$person_ctry_code == "US"], use = "complete.obs")
cor(df$VA_share[df$person_ctry_code == "US"], df$process_ratio_5_ind[df$person_ctry_code == "US"], use = "complete.obs")

print('Germany')
cor(df$VA_share[df$person_ctry_code == "DE"], df$process_ratio_2_ind[df$person_ctry_code == "DE"], use = "complete.obs")
cor(df$VA_share[df$person_ctry_code == "DE"], df$process_ratio_5_ind[df$person_ctry_code == "DE"], use = "complete.obs")

print('Romania')
cor(df$VA_share[df$person_ctry_code == "RO"], df$process_ratio_2_ind[df$person_ctry_code == "RO"], use = "complete.obs")
cor(df$VA_share[df$person_ctry_code == "RO"], df$process_ratio_5_ind[df$person_ctry_code == "RO"], use = "complete.obs")

print('China')
cor(df$VA_share[df$person_ctry_code == "CN"], df$process_ratio_2_ind[df$person_ctry_code == "CN"], use = "complete.obs")
cor(df$VA_share[df$person_ctry_code == "CN"], df$process_ratio_5_ind[df$person_ctry_code == "CN"], use = "complete.obs")


# C-I data----

#change some variables
df$titleP_bin <- 0
df$titleP_bin[which(df$title_process == "t")] <- 1
df$abstractP_bin <- 0
df$abstractP_bin[which(df$abstract_process == "t")] <- 1
df$titleU_bin <- 0
df$titleU_bin[which(df$title_use == "t")] <- 1
df$abstractU_bin <- 0
df$abstractU_bin[which(df$abstract_use == "t")] <- 1
dfd <- fastDummies::dummy_cols(df, select_columns = c("Y02PN"))
dfd[,startsWith(colnames(dfd), "Y02PN_Y02")][is.na(dfd[,startsWith(colnames(dfd), "Y02PN_Y02")])] <- 0

dfd$pu_share_2ind <- dfd$process_ratio_2_ind +dfd$use_ratio_ind
dfd$pu_share_5ind <- dfd$process_ratio_5_ind+ dfd$use_ratio_ind

# number of patents in each c-i pair that have process-share data for weighted process share variables:
denominator2 <- dfd %>% 
  group_by(earliest_filing_year, person_ctry_code, IndustryCode) %>% 
  filter(!is.na(process_ratio_2_ind)) %>% 
  mutate(sum_weights = sum(weight)) %>% 
  select(earliest_filing_year, person_ctry_code, IndustryCode, sum_weights) %>% 
  distinct()
denominator5 <- dfd %>% 
  group_by(earliest_filing_year, person_ctry_code, IndustryCode) %>% 
  filter(!is.na(process_ratio_5_ind)) %>% 
  mutate(sum_weights = sum(weight)) %>% 
  select(earliest_filing_year, person_ctry_code, IndustryCode, sum_weights) %>% 
  distinct()
denominator2pu <- dfd %>% 
  group_by(earliest_filing_year, person_ctry_code, IndustryCode) %>% 
  filter(!is.na(pu_share_2ind)) %>% 
  mutate(sum_weights = sum(weight)) %>% 
  select(earliest_filing_year, person_ctry_code, IndustryCode, sum_weights) %>% 
  distinct()
denominator5pu <- dfd %>% 
  group_by(earliest_filing_year, person_ctry_code, IndustryCode) %>% 
  filter(!is.na(pu_share_5ind)) %>% 
  mutate(sum_weights = sum(weight)) %>% 
  select(earliest_filing_year, person_ctry_code, IndustryCode, sum_weights) %>% 
  distinct()

#create dataset
dfd <- dfd %>% 
  # group by observation level variables
  group_by(earliest_filing_year, person_ctry_code, 
           IndustryCode) %>% 
  select(appln_id, VA_share, labsh, rgdpo, iso_alpha3, docdb_family_size, 
         nb_applicants, nb_inventors, eu_member, 
         Y02PN, titleP_bin, abstractP_bin, titleU_bin, abstractU_bin, 
         process_ratio_2_ind, process_ratio_5_ind, pu_share_2ind, pu_share_5ind,
         weight, IndustryDescription, VA, GO, hc, pop, rtfpna, emp,
         Y02PN_Y02P1,Y02PN_Y02P2, Y02PN_Y02P3, Y02PN_Y02P4, Y02PN_Y02P6, 
         Y02PN_Y02P7, Y02PN_Y02P8, Y02PN_Y02P9, Y02PN_NA) %>%
  distinct() %>% 
  #process patents 0/1 times weight
  mutate(PP_title = weight*titleP_bin) %>% 
  mutate(PP_abstract = weight*abstractP_bin) %>% 
  #use patents 0/1 times weight
  mutate(UP_title = weight*titleU_bin) %>% 
  mutate(UP_abstract = weight*abstractU_bin) %>% 
  # mean of process_share across C-I pairs
  mutate(Pshare_mean = mean(process_ratio_2_ind, na.rm = T)) %>%
  mutate(Pshare5_mean = mean(process_ratio_5_ind, na.rm = T)) %>%
  mutate(pu_share_2ind_mean = mean(pu_share_2ind, na.rm = T)) %>%
  mutate(pu_share_5ind_mean = mean(pu_share_5ind, na.rm = T)) %>%
  mutate(Pshare_mean_weighted = sum(weight*process_ratio_2_ind, na.rm = T)) %>% 
  mutate(Pshare_mean5_weighted = sum(weight*process_ratio_5_ind, na.rm = T)) %>% 
  mutate(pu_share_2ind_mean_weighted = sum(weight*pu_share_2ind, na.rm = T)) %>% 
  mutate(pu_share_5ind_mean_weighted = sum(weight*pu_share_5ind, na.rm = T)) %>% 
  # Y02P
  mutate(Y02PN_Y02P1 = weight*Y02PN_Y02P1) %>% 
  mutate(Y02PN_Y02P1 = weight*Y02PN_Y02P1) %>% 
  mutate(Y02PN_Y02P2 = weight*Y02PN_Y02P2) %>% 
  mutate(Y02PN_Y02P3 = weight*Y02PN_Y02P3) %>% 
  mutate(Y02PN_Y02P4 = weight*Y02PN_Y02P4) %>% 
  mutate(Y02PN_Y02P6 = weight*Y02PN_Y02P6) %>% 
  mutate(Y02PN_Y02P7 = weight*Y02PN_Y02P7) %>% 
  mutate(Y02PN_Y02P8 = weight*Y02PN_Y02P8) %>% 
  mutate(Y02PN_Y02P9 = weight*Y02PN_Y02P9) %>% 
  mutate(Y02PN_NA = weight*Y02PN_NA) %>% 
  # sum number of patents over C-I pairs per year:
  ungroup() %>% 
  group_by(earliest_filing_year, person_ctry_code, IndustryCode) %>% 
  mutate(PA_c_i = sum(weight)) %>%
  mutate(PP_title_c_i = sum(PP_title)) %>%
  mutate(PP_abstract_c_i = sum(PP_abstract)) %>%
  mutate(UP_title_c_i = sum(UP_title)) %>%
  mutate(UP_abstract_c_i = sum(UP_abstract)) %>%
  mutate(Y02PN_Y02P1_c_i = sum(Y02PN_Y02P1)) %>%
  mutate(Y02PN_Y02P2_c_i = sum(Y02PN_Y02P2)) %>%
  mutate(Y02PN_Y02P3_c_i = sum(Y02PN_Y02P3)) %>%
  mutate(Y02PN_Y02P4_c_i = sum(Y02PN_Y02P4)) %>%
  mutate(Y02PN_Y02P6_c_i = sum(Y02PN_Y02P6)) %>%
  mutate(Y02PN_Y02P7_c_i = sum(Y02PN_Y02P7)) %>%
  mutate(Y02PN_Y02P8_c_i = sum(Y02PN_Y02P8)) %>%
  mutate(Y02PN_Y02P9_c_i = sum(Y02PN_Y02P9)) %>%
  mutate(Y02PN_NA_c_i = sum(Y02PN_NA)) %>%
  ungroup() %>% 
  group_by(earliest_filing_year, person_ctry_code) %>% 
  mutate(PA_c = sum(weight)) %>% 
  ungroup() %>% 
  group_by(earliest_filing_year, IndustryCode) %>% 
  mutate(PA_industry = sum(weight)) %>% 
  ungroup() %>% 
  group_by(earliest_filing_year) %>% 
  mutate(PA_global = sum(weight)) %>% 
  ungroup() %>% 
  select(-appln_id, -docdb_family_size, -nb_applicants, -nb_inventors, 
         -Y02PN, -titleP_bin, -abstractP_bin, -titleU_bin, -abstractU_bin, 
         -process_ratio_5_ind, -process_ratio_2_ind, -weight, -pu_share_2ind, -pu_share_5ind,
         -PP_title, -PP_abstract, -UP_title, -UP_abstract,
         -Y02PN_NA, -Y02PN_Y02P1, -Y02PN_Y02P2, -Y02PN_Y02P3, -Y02PN_Y02P4, -Y02PN_Y02P6, 
         -Y02PN_Y02P7, -Y02PN_Y02P8, -Y02PN_Y02P9, ) %>% 
  distinct() %>% 
  arrange(person_ctry_code, earliest_filing_year, IndustryCode)


# get the weighted process-share variables:

dfd <- merge(dfd, denominator2, 
             by = c("earliest_filing_year", "person_ctry_code", "IndustryCode"))

dfd$Pshare_mean_weighted <- dfd$Pshare_mean_weighted/dfd$sum_weights
dfd <- dfd %>% select(-sum_weights)  

dfd <- merge(dfd, denominator5, 
             by = c("earliest_filing_year", "person_ctry_code", "IndustryCode"))

dfd$Pshare_mean5_weighted <- dfd$Pshare_mean5_weighted/dfd$sum_weights
dfd <- dfd %>% select(-sum_weights) 

dfd <- merge(dfd, denominator2pu, 
             by = c("earliest_filing_year", "person_ctry_code", "IndustryCode"))

dfd$pu_share_2ind_mean_weighted <- dfd$pu_share_2ind_mean_weighted/dfd$sum_weights
dfd <- dfd %>% select(-sum_weights)  

dfd <- merge(dfd, denominator5pu, 
             by = c("earliest_filing_year", "person_ctry_code", "IndustryCode"))

dfd$pu_share_5ind_mean_weighted <- dfd$pu_share_5ind_mean_weighted/dfd$sum_weights
dfd <- dfd %>% select(-sum_weights)  

rm(denominator2, denominator5, denominator2pu, denominator5pu)

# some countries do not apply for any Y02-patents in some years? --NO: we have a balanced panel!
cbind(
  # number of countries per year 
  dfd %>% group_by(earliest_filing_year) %>% select(person_ctry_code) %>% distinct() %>% count() %>% rename(countries = n),
  # number of industries per year
  dfd %>% group_by(earliest_filing_year) %>% select(IndustryCode) %>% distinct() %>% count() %>% 
    ungroup() %>% select(-earliest_filing_year) %>% rename(industries = n)
)



# Export Data----
write.csv(dfd, "D:/Tinnefeld-user6/Data/6_allPatents_balancedPanel.csv")
